
#include "dmx.h"

// init DMX interface
void init_dmx(int ch)
{
	LED_DDR |= (1<<LED);						// status LED
	LED_PORT |= (1<<LED);
	
	DmxAddress = ch;							// set start address
	
	unsigned int ubrr = 1;						// 250kBaud
	UBRRH = (unsigned char)(ubrr>>8);
	UBRRL = (unsigned char)ubrr;

	UCSRC = (1<<URSEL)|(3<<UCSZ0)|(1<<USBS);	// 8n2
	UCSRB = (1<<RXEN)|(1<<RXCIE);
	
	gDmxState = IDLE;
}

// DMX reception interrupt
ISR(USART_RXC_vect)
{
	static uint16_t DmxCount;
	uint8_t USARTstate = UCSRA;				// get state before data!
	uint8_t DmxByte = UDR;					// get data
	uint8_t DmxState = gDmxState;			// just load once from SRAM to increase speed
	
	if(USARTstate & (1<<FE))				// check for break
	{
		UCSRA &= ~(1<<FE);					// reset flag (necessary for simulation in AVR Studio)
		DmxCount = DmxAddress;				// reset channel counter (count channels before start address)
		gDmxState = BREAK;
	}
	else if(DmxState == BREAK)
	{
		if(DmxByte == 0)
			gDmxState = STARTB;				// normal start code detected
		else
			gDmxState = IDLE;
	}
	else if(DmxState == STARTB)
	{
		if(--DmxCount == 0)					// start address reached?
		{
			DmxCount = 1;					// set up counter for required channels
			DmxRxField[0] = DmxByte;		// get 1st DMX channel of device
			gDmxState = STARTADR;
		}
		LED_PORT ^= (1<<LED);
	}
	else if(DmxState == STARTADR)
	{
		DmxRxField[DmxCount++] = DmxByte;	// get channel
		if(DmxCount >= sizeof(DmxRxField)) 	// all ch received?
			gDmxState = IDLE;				// wait for next break
	}				
}
